// ClientComplianceMain.cpp 
// Console application
// Sample c++ application that statically links against ClientComplianceLib

#include "ClientCompliance.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#ifdef _WIN32
#include <windows.h>
#define sleep(x) Sleep(x*1000)
#else
#include <unistd.h>
#endif

void PrintUsageMessage()
{
	const char *progName = "ClientCompliance";
	printf("Usage: \n");
	printf("       %s -d\n", progName);	// print diagnostic information only
	printf("       %s -c <url> <doc> [<numeric flags>]\n", progName);
	printf("  where:\n");
	printf("    <url> is a site gather url the BESClient is subscribed.\n");
	printf("    <doc> is the name of the compliance document to process.\n");
	printf("  sample:\n");
	printf("    %s -c http://myserver:52311/cgi-bin/bfgather.exe/actionsite ComplianceDoc.xml\n", progName);
	printf("  note:\n");
	printf("    in this example <doc> would be in the __BESData\\actionsite\\__Compliance folder.\n");
	printf("    flags = a number from 0 to 7 (the sum of the following values).\n");
	printf("          = 1 - COMPLIANCE_FLAG_GATHER         - gather all sites.\n");
	printf("          = 2 - COMPLIANCE_FLAG_EVALUATE_ALL   - evaluate all fixlets.\n");
	printf("          = 4 - COMPLIANCE_FLAG_IDLE_CPU_USAGE - use IDLE cpu resource level.\n");
	printf("          = 8 - COMPLIANCE_FLAG_ACTIONHINT     - treat <doc> arg as action id hint.\n");
}

void ProcessRequest(const char *siteURL, const char *complianceDocument, unsigned int flag)
{

	int openError = COMPLIANCE_Open( siteURL, complianceDocument, flag );

	if( openError != 0 )
	{
		printf( "COMPLIANCE_Open returned error: %d.\n", openError );
	}
	else
	{
		// If open succeeds, make sure to call COMPLIANCE_Close
		// to free up resources, but not until you are done looking at results.

		int progress;
		unsigned int error;

		do
		{
			unsigned int progressPercent;

			progress = COMPLIANCE_Progress( &progressPercent, &error );

			if( progress == COMPLIANCE_PROGRESS_BUSY )
			{
				printf( "COMPLIANCE_Progress %d%% completed.\n", progressPercent );
				sleep(1);
			}

		} while ( progress == COMPLIANCE_PROGRESS_BUSY );


		if( progress == COMPLIANCE_PROGRESS_COMPLETE )
		{
			int count = COMPLIANCE_ResultCount();

			printf("COMPLIANCE_ResultCount is %d.\n", count);

			for( int index=0; index< count ;index++ )
			{
				const char *designator;  // designator of value found
				const char *result;      // result of evaluating relevance
				const char *resultType;
				const char *evalTime; 
				const char *description; // description accompanying designator
				const char *comment;     // comment accompanying designator

				if( COMPLIANCE_IndexedValue( index,
											 &designator,
											 &result,
											 &resultType,
											 &evalTime,
											 &description,
											 &comment ) == 0)
				{
					printf("    Designator: %s\n", designator);
					printf("        Result: %s\n", result);
					printf("   Result Type: %s\n", resultType);
					printf("Eval Time (s): %s\n", evalTime);
					printf("          Desc: %s\n", description);
					printf("       Comment: %s\n", comment);
					printf("\n");
				}
			}
		}
		else if( progress == COMPLIANCE_PROGRESS_ERROR )
		{
			if( error == COMPLIANCE_ERROR_CHANGING_CONDITIONS )
				printf( "Client reports not available right now.\n" );
			else
				printf( "COMPLIANCE_Progress returned error %d.\n", error );
		}

		COMPLIANCE_Close();

	}
}

int main(int argc, char* argv[])
{

	if( argc == 2 && !strcmp(argv[1], "-d") )
	{
		ProcessRequest( 0, 0, 0 ); // Diagnostic  request
		return 0;
	}
	else if( (argc == 4 || argc == 5 ) && !strcmp(argv[1], "-c") )
	{
		const char *siteURL = argv[2];
		const char *complianceDocument = argv[3];
		unsigned int flags = 0;
		if( argc == 5 )
			flags = atoi( argv[4] );
		if( flags <= (
			COMPLIANCE_FLAG_GATHER         | 
			COMPLIANCE_FLAG_EVALUATE_ALL   | 
			COMPLIANCE_FLAG_IDLE_CPU_USAGE |
			COMPLIANCE_FLAG_ACTIONHINT      ) 
			)
		{
			ProcessRequest( siteURL, complianceDocument, flags );
			return 0;
		}
	}
	// usage message
	PrintUsageMessage();
	return -1;

}

